"""Rich console helpers for consistent terminal output."""

from rich.console import Console
from rich.theme import Theme

ievo_theme = Theme(
    {
        "success": "#4ade80",
        "error": "#f87171",
        "warn": "#fbbf24",
        "info": "#22d3ee",
        "dim": "#555555",
        "agent": "#a78bfa",
        "version": "dim #22d3ee",
        "bright": "#e0e0e0",
        "accent": "#4ade80",
        "opus": "#f87171",
        "sonnet": "#a78bfa",
        "haiku": "#22d3ee",
        "blue": "#60a5fa",
    }
)

console = Console(theme=ievo_theme)


def success(msg: str) -> None:
    console.print(f"[success]✓[/success] {msg}")


def error(msg: str) -> None:
    console.print(f"[error]✗[/error] {msg}")


def warn(msg: str) -> None:
    console.print(f"[warn]![/warn] {msg}")


def info(msg: str) -> None:
    console.print(f"[info]⚡[/info] {msg}")


def step(msg: str) -> None:
    console.print(f"  [dim]→[/dim] {msg}")
